package backend

import (
	"cron_admin/app/constants/errorcode"
	"cron_admin/app/http/controllers"
	"cron_admin/app/http/entities/backend"
	"cron_admin/app/models/cronusermodel"
	"cron_admin/app/services"
	userServices "cron_admin/app/services/user_service"
	"cron_admin/app/utils/helper"
	"cron_admin/app/utils/mapstructure"
	"github.com/gin-gonic/gin"
	"strings"
)

func CronReportLogsList(c *gin.Context) {
	request := controllers.GetRequest(c).(*backend.CronReportLogsListRequest)
	count, reportLogsList, err := services.CronReportLogsList(request, request.Page, request.Limit)
	if err != nil {
		controllers.HandRes(c, nil, errorcode.ParamError)
	} else {
		var reportLogsListResponse []backend.CronReportLogsListResponse
		_ = mapstructure.DecodeWithTime(reportLogsList, &reportLogsListResponse, helper.DefaultFormatLayout)
		//	合并reportLogsListResponse下的所有user_id 并去重
		var userIds []string
		userIdsMap := make(map[int64]bool)
		for _, v := range reportLogsListResponse {
			userIdsStr := strings.Split(v.UserId, ",")
			for _, userId := range userIdsStr {
				if userId != "" {
					if _, ok := userIdsMap[helper.StringToInt64(userId)]; ok {
						continue
					}
					userIdsMap[helper.StringToInt64(userId)] = true
					userIds = append(userIds, userId)
				}
			}
		}
		_, userList, err := userServices.GetListByWhere(&backend.UserListRequest{UserIds: userIds}, 1, 100)
		if err != nil {
			return
		}
		userListMap := make(map[int64]cronusermodel.CronUser)
		for _, v := range userList {
			userListMap[int64(v.UserId)] = v
		}
		for k, v := range reportLogsListResponse {
			userIdsStr := strings.Split(v.UserId, ",")
			var userNames []string
			for _, userId := range userIdsStr {
				if userId != "" {
					userNames = append(userNames, userListMap[helper.StringToInt64(userId)].Name)
				}
			}
			userNamesStr := strings.Join(userNames, ",")
			reportLogsListResponse[k].UserName = userNamesStr
		}

		controllers.HandRes(c, gin.H{"data": reportLogsListResponse, "count": count}, err)
	}
}
